// ------------------------------------------------------------------
//																		
// Purpose.............:	UrlList
// Created.............:	May 8, 2001
// Copyright...........:	(c) 2001 by Adobe Systems
//
// ------------------------------------------------------------------

import java.io.*;
import java.net.URL;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.net.URLDecoder;
import org.apache.regexp.*;
import java.util.ResourceBundle;
import java.util.Locale;

/**
 * This class stores specified URL and saving filename from URL list file.
 * For example:
 * <pre>
 *   UrlList urlList = new UrlList();
 *   urlList.readFile(fileName, skipFolder);
 * </pre>
 *
 * @author  Adobe Systems, Inc.
 * @since 1.0
 */
public class UrlList {

	/** It stores number of records. */
	static int		count;

	/** It stores Url classes. */
	static Url[]	url;

	/** It stores filenames. */
	static String[] file;

	/** It stores saving filepath. */
	static String[] filePath;


	/**
	 * Initialize class variables.
	 * @param	void
	 * @return	void
	 */
	public void init() {
		count	 = 0;
		url      = new Url[5000];
		file     = new String[5000];
		filePath = new String[5000];
	}

	/**
	 * Read URL list file
	 * @param	fileName Specify URL file path
	 * @return	void
	 */
	public void readFile(String fileName) throws IOException {
		readFile(fileName, 0);
	}

	/**
	 * Read URL list file
	 * @param	fileName Specify URL file path
	 * @param	skipFolder Specify how many folder to skip
	 * @return	void
	 */
	public void readFile(String fileName, int skipFolder) throws IOException {
		init();

		final String httpStr = "http://";
		final String fileStr = "file://";
		BufferedInputStream	bis = null;
		HttpURLConnection	http = null;

		String content = "";
		try {
			if (0 == httpStr.compareToIgnoreCase(fileName.substring(0, httpStr.length()))) {
				// http://.....
				URL url = new URL(fileName);
				http = (HttpURLConnection)url.openConnection();
				bis = new BufferedInputStream(http.getInputStream());
			} else if (0 == fileStr.compareToIgnoreCase(fileName.substring(0, fileStr.length()))) {
				// file://.....
				String filePath = fileName.substring(5);
				filePath = filePath.replace('/', File.separatorChar);
				bis = new BufferedInputStream(new FileInputStream(filePath));
			} else {
				bis = new BufferedInputStream(new FileInputStream(fileName));
			}

			byte[] b = new byte[32768];
			StringBuffer sb = new StringBuffer("");
			int size;
			while ((size = bis.read(b)) != -1) {
				sb.append(new String(b, 0, size));
			}
			content = sb.toString();
		} catch(IOException e) {
			ResourceBundle rb = ResourceBundle.getBundle("PageGenerator");
			System.err.println(rb.getString("error") + e.getMessage());
			if (http != null) {
				System.err.println(rb.getString("httpError") + http.getResponseCode() + " " + http.getResponseMessage());
			}
			e.printStackTrace();
			System.exit(-1);
		} finally {
		}

	if (content.length() == 0) {
		System.exit(-1);
	}


		RE reRowBegin = null, reRowDelimit = null, reRowEnd = null;
		RE reDataDelimit = null, reDataBegin = null, reSeparater = null;
		RE reDelimiter = null, reDataEnd = null, reLineEnd = null, reAmp = null;
		RE reTrim = null, reTableBegin;

		String	separater = "<SEPARATER>";	// separater between each URL
		String	delimiter = "<DELIMITER>";	// delimitter between parameter (URL, filename, etc.)
		String	dataBegin = "<DATABEGIN>";	// before this, there's junk data
		String	dataEnd   = "<DATAEND>";	// after this, there's junk data
		String	dataReset = "<DATARESET>";	// data reset for supporting multiple table in a page

		try {
			reSeparater = new RE(separater, RE.MATCH_MULTILINE);
			reDelimiter = new RE(delimiter, RE.MATCH_MULTILINE);
			reAmp		= new RE("&amp;", RE.MATCH_MULTILINE);
			reTrim		= new RE("("+delimiter+")*"+separater, RE.MATCH_MULTILINE);

			content = reAmp.subst(content, "&");	// this is for copied from HTML table from IE

			if (fileName.endsWith(".txt")) {	// URL list is text file
				reLineEnd = new RE("(\\r|\\n)+", RE.MATCH_MULTILINE);
				reDataDelimit = new RE("(\\s)+", RE.MATCH_MULTILINE);

				content = reLineEnd.subst(content, separater);
				content = reDataDelimit.subst(content, delimiter);
				content = reTrim.subst(content, separater);	// trimming line end space
				content = separater + content + separater;
			} else {	// URL list is HTML table
				reTableBegin  = new RE("<[Tt][Aa][Bb][Ll][Ee](\\s+[^>]*)*?>(\\s+[^<]*)",  RE.MATCH_MULTILINE);
				reRowBegin    = new RE("<[Tt][Rr](\\s+[^>]*)*?>\\s*<[Tt][Dd](\\s+[^>]*)*?>",  RE.MATCH_MULTILINE);
				reRowDelimit  = new RE("</[Tt][Dd](\\s+[^>]*)*?>\\s*<[Tt][Dd](\\s+[^>]*)*?>", RE.MATCH_MULTILINE);
				reRowEnd	  = new RE("</[Tt][Dd](\\s+[^>]*)*?>\\s*</[Tt][Rr](\\s+[^>]*)*?>",RE.MATCH_MULTILINE);
				reDataDelimit = new RE(dataEnd+"\\s*"+dataBegin, RE.MATCH_MULTILINE);
				reDataBegin	  = new RE(dataBegin,	RE.MATCH_MULTILINE);
				reDataEnd	  = new RE(dataEnd,		RE.MATCH_MULTILINE);

				content = reTableBegin.subst(content, separater + dataReset);	// starting table == expect different format
				content = reRowBegin.subst(content, dataBegin);
				content = reRowDelimit.subst(content, delimiter);
				content = reRowEnd.subst(content, dataEnd);
				content = reDataDelimit.subst(content, separater);
				content = reDataBegin.subst(content, separater);
				content = reDataEnd.subst(content, separater);
				content = reSeparater.subst(content, separater);
			}
		} catch (RESyntaxException e) {
			e.printStackTrace();
		}

		String[] rows = reSeparater.split(content);
		boolean firstData = true;
		String[] argNames = new String[20];
		int argNumber = 0;
		for (int row = 1; row < rows.length -1; row++) {
			if (rows[row].compareTo(dataReset) == 0) {	// dataReset == different format
				firstData = true;
				continue;
			}
			if (rows[row].indexOf(delimiter) == -1) {		// no delimiter == junk data
				continue;
			}
			String[] cells = reDelimiter.split(rows[row]);

			if (firstData) {
				firstData = false;
				if (2 < cells.length) {
					argNumber = cells.length - 2;
					for (int i = 0; i < argNumber; i++) {
						argNames[i] = new String(cells[i+1].trim());
					}
					continue;
				}
			}

			String specifiedUrl  = cells[0].trim();
			String addtionalParameter = "";
			for (int i = 0; i < argNumber; i++) {
				if (i != 0)	addtionalParameter += '&';
				addtionalParameter += argNames[i] + '=' + URLEncoder.encode(cells[i+1].trim());
			}
			String specifiedFile = cells[cells.length-1].trim();

			if ((specifiedUrl.length() == 0)||(specifiedFile.length() == 0)) {
				continue;
			}

			if (addtionalParameter.length() != 0) {
				char lastChar = specifiedUrl.charAt(specifiedUrl.length()-1);
				if ((lastChar == '&')||(lastChar == '?')) {
					specifiedUrl += addtionalParameter;
				} else {
					if (specifiedUrl.indexOf("?") == -1)	specifiedUrl += "?" + addtionalParameter;
					else								specifiedUrl += "&" + addtionalParameter;
				}
			}

			url[count] = new Url();
			url[count].setUrl(specifiedUrl);

			specifiedFile.replace(File.separatorChar, '/');
			int cutIndex = 0;
			String addtionalPath = "";
			if (-1 == (cutIndex = specifiedFile.lastIndexOf("/"))) {	//	no subdirectory is specified
				file[count]	= specifiedFile;
			} else {
				addtionalPath = specifiedFile.substring(0, cutIndex);
				file[count]	= specifiedFile.substring(cutIndex+1);
			}

			int iRootSlash = specifiedUrl.indexOf('/', 8);
			int iLastSlash = specifiedUrl.lastIndexOf('/');
			if ((iRootSlash == -1)||(iLastSlash == -1)) {
				ResourceBundle rb = ResourceBundle.getBundle("PageGenerator");
				System.err.println(rb.getString("wrongUrlFormat") + specifiedUrl);
				continue;
			}
			
			String path = specifiedUrl.substring(iRootSlash+1, iLastSlash+1);
			if (addtionalPath.length() != 0) {
				path += addtionalPath + "/";
			}
			int iSkipper = 0;
			for (int i = 0; i < skipFolder; i++) {
				if (-1 == (iSkipper = path.indexOf('/', iSkipper+1))) {
					path = "";
					break;
				}
				path = path.substring(iSkipper+1);
			}
			filePath[count] = path.replace('/', File.separatorChar);
			count++;
		}
//		print();	// Here's good place to check URL list file.
	}

	/**
	 * Create full URL from link
	 * @param	index Index to URL array
	 * @param	orgUrl Original URL
	 * @return	URL string
	 */
	static String getFullUrlFromLink(int index, String orgUrl) {
		Url currentUrl = url[index];
		String replaceFileName = "";
		int climbUp = 0;
		String orgUrlFull = "";
		String http = "http";
		String parent = "../";

		if (0 == http.compareToIgnoreCase(orgUrl.substring(0, http.length()))) {	// It's full URL.
			orgUrlFull = orgUrl;
		} else {
			if ('/' == orgUrl.charAt(0)) {	//	stating "/" URL
				orgUrlFull = currentUrl.dir[0] + orgUrl;
			} else if (0 == parent.compareTo(orgUrl.substring(0, parent.length()))) {	//	"../" case
				int searchIndex = parent.length();
				int lastIndex = searchIndex;
				climbUp = 1;
				while (-1 != (searchIndex = orgUrl.indexOf(parent, searchIndex+1))) {
					climbUp++;
					lastIndex = searchIndex;
				}
				if (currentUrl.level < climbUp) {
					ResourceBundle rb = ResourceBundle.getBundle("PageGenerator");
					System.err.println(rb.getString("error") + rb.getString("wrongParentPath"));
					return "";
				}
				String path = "";
				for (int i = 0; i < climbUp; i++) {
					path += currentUrl.dir[i] + "/";
				}
				orgUrlFull = path + orgUrl;
			} else if (('.' == orgUrl.charAt(0)&&('/' == orgUrl.charAt(1)))) {	//	stating "./" URL
				String path = "";
				for (int i = 0; i < currentUrl.level; i++) {
					path += currentUrl.dir[i] + "/";
				}
				orgUrlFull = path + orgUrl.substring(2);
			} else {
				String path = "";
				for (int i = 0; i < currentUrl.level; i++) {
					path += currentUrl.dir[i] + "/";
				}
				orgUrlFull = path + orgUrl;
			}
		}
		return orgUrlFull;
	}

	/**
	 * Find matching URL and returns index
	 * @param	findingUrl URL looking for
	 * @return	Index to found file. It returns -1 when the file is not found.
	 */
	private static int findReplaceFile(String findingUrl) {
		String decoded = "";
		try {
			decoded = URLDecoder.decode(findingUrl);
		} catch(Exception e) {
			e.printStackTrace();
		}

		for (int i = 0; i < count; i++) {
			if (0 == decoded.compareTo(url[i].decodedUrl)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Make relative path from 2 URLs.
	 * @param	from From URL index
	 * @param	to   To   URL index
	 * @return	Relative URL path string
	 */
	private static String getDeltaPath(int from, int to) {
		int common = 0;
		String delta = "";

		for (int i = 0; ((i < url[from].level)&&(i < url[to].level)); i++) {
			if (0 != url[from].dir[i].compareTo(url[to].dir[i]))	break;
			common++;
		}
		for (int i = common; i < url[from].level; i++)	delta += "../";
		for (int i = common; i < url[to].level;   i++)	delta += url[to].dir[i] + "/";
		return delta;
	}

	/**
	 * Get replace file path
	 * @param	index Index to the URL
	 * @param	orgUrl Original URL string
	 * @return	Path string. It returns "" when error occurs.
	 */
	public static String getReplaceFilePath(int index, String orgUrl) {
		String orgUrlFull = getFullUrlFromLink(index, orgUrl);
		if ("" == orgUrlFull) {
			ResourceBundle rb = ResourceBundle.getBundle("PageGenerator");
			System.err.println(rb.getString("unexpectedError"));
			// orgUrlFull = NULL
			return "";
		}
		int replaceIndex = findReplaceFile(orgUrlFull);
		if (-1 == replaceIndex) {
			// URL [" + orgUrlFull + "] was not found in the list.
			return "";
		}
		String path = getDeltaPath(index, replaceIndex);
		return path + file[replaceIndex];
	}

	/**
	 * Returns url without parameter
	 * @param	url URL string
	 * @return	URL string
	 */
	private static String cutUrlParameter(String url) {
		int	index = url.indexOf('?');
		if (index == -1)	return url;
		else				return url.substring(0, index);
	}

	/**
	 * Display all class variables
	 * @param	void
	 * @return	void
	 */
	public static void print() {
		ResourceBundle rb = ResourceBundle.getBundle("PageGenerator");
		print(rb.getString("urlCount") + count);
		for (int i = 0; i < count; i++) {
			System.out.println("[" + i + "]");
			url[i].print();
			System.out.println("file      = [" + file[i]       + "]");
			System.out.println("filePath  = [" + filePath[i]   + "]");
		}
	}

	/**
	 * Display string
	 * @param	s String to display
	 * @return	void
	 */
	private static void print(String s) {
		System.out.println(s);
	}
}

